/**
 * <one line to give the library's name and a brief idea of what it does.>
 * Copyright (C) 2005 Andy Turner, CCG, University of Leeds, UK
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.ac.leeds.ccg.andyt.projects.geomorphometrics;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashSet;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellAbstract;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDouble;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunk64CellMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkAbstractFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkArrayFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkJAIFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkRAFFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellInt;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunk64CellMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkAbstractFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkArrayFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkJAIFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkRAFFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntFactory;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatistics0;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatistics1;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatisticsAbstract;
import uk.ac.leeds.ccg.andyt.grids.exchange.ImageExporter;
import uk.ac.leeds.ccg.andyt.grids.exchange.ESRIAsciiGridExporter;
import uk.ac.leeds.ccg.andyt.grids.process.Grid2DSquareCellProcessorDEM;
import uk.ac.leeds.ccg.andyt.grids.utilities.Utilities;
import uk.ac.leeds.ccg.andyt.grids.utilities.FileCreator;

public  class Geog5060_prac7
        extends Grid2DSquareCellProcessorDEM {
    
    private long time;
    
    /** Creates a new Geog5060_prac7 */
    public Geog5060_prac7() {
        this( FileCreator.createNewFile() );
    }
    
    /**
     * Creates a new Geog5060_prac7 using specified workspace.
     * WARNING: Files in the specified workspace may get overwritten.
     */
    public Geog5060_prac7( File workspace ) {
        super( workspace );
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        File workspace = new File( "M:/geog5060_prac7/" );
        Geog5060_prac7 aGeog5060_prac7 = new Geog5060_prac7( workspace );
        aGeog5060_prac7.run();
    }
    
    public void run() {
        try {
            time = System.currentTimeMillis();
            boolean handleOutOfMemoryError = true;
            runTest( handleOutOfMemoryError );
            System.out.println(
                    "Processing complete in " +
                    Utilities.time( System.currentTimeMillis() - time ) );
        } catch ( Exception e ) {
            e.printStackTrace();
        } catch ( Error e ) {
            e.printStackTrace();
        }
    }
    
    public void runTest( boolean handleOutOfMemoryError ) throws Exception {
        log(    "runTest();" ,
                handleOutOfMemoryError );
        
        String filename = "catchment1.asc";
        String dataDirectory = this.workspace.toString() + "/data/";
        
        String filenameWithoutExtension = filename.substring( 0, filename.length() - 4 );
        time = System.currentTimeMillis();
        int chunkNrows = 256; //64, 128, 256, 512, 1024, 2048
        int chunkNcols = 256; //64, 128, 256, 512, 1024, 2048
        
        this.grid2DSquareCellDoubleFactory.setChunkNrows( chunkNrows );
        this.grid2DSquareCellDoubleFactory.setChunkNcols( chunkNcols );
        this.grid2DSquareCellDoubleFactory.setHandleOutOfMemoryError( handleOutOfMemoryError );
        
        this.grid2DSquareCellIntFactory.setChunkNrows( chunkNrows );
        this.grid2DSquareCellIntFactory.setChunkNcols( chunkNcols );
        this.grid2DSquareCellIntFactory.setHandleOutOfMemoryError( handleOutOfMemoryError );
        
        Grid2DSquareCellInt grid2DSquareCellInt = null;
        Grid2DSquareCellDouble grid2DSquareCellDouble = null;
        
        boolean load1 = true;
        if ( load1 ) {
            File file = new File( dataDirectory + filename );
            //grid2DSquareCellInt = ( Grid2DSquareCellInt ) grid2DSquareCellIntFactory.create( file );
            grid2DSquareCellDouble = ( Grid2DSquareCellDouble ) grid2DSquareCellDoubleFactory.create( file );
            boolean swapToFileCache = true;
            //grid2DSquareCellInt.writeToFile(
            grid2DSquareCellDouble.writeToFile(
                    swapToFileCache,
                    handleOutOfMemoryError );
        }
        grid2DSquareCellDouble = ( Grid2DSquareCellDouble ) grid2DSquareCellDoubleFactory.create( new File( dataDirectory + filenameWithoutExtension + "uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleFactory_chunkNrows(" + chunkNrows + ")_chunkNcols(" + chunkNcols + ")" ) );
        
        new ImageExporter().toGreyScaleImage(
                grid2DSquareCellDouble,
                new File(
                dataDirectory,
                grid2DSquareCellDouble.getName( handleOutOfMemoryError ) + ".png" ),
                "PNG",
                handleOutOfMemoryError );
        
        doMetrics1(
                this.grid2DSquareCellDoubleFactory,
                this.grid2DSquareCellIntFactory,
                grid2DSquareCellDouble,
                new File( dataDirectory ),
                handleOutOfMemoryError );
        
        log(    "Processing complete in " + Utilities.time( System.currentTimeMillis() - time ),
                handleOutOfMemoryError );
    }
    
    public void doMetrics1(
            Grid2DSquareCellDoubleFactory grid2DSquareCellDoubleFactory,
            Grid2DSquareCellIntFactory grid2DSquareCellIntFactory,
            Grid2DSquareCellAbstract grid2DSquareCell,
            File dataDirectory,
            boolean handleOutOfMemoryError ) {
        // Initialistaion
        BigDecimal[] dimensions = grid2DSquareCell.getDimensions( handleOutOfMemoryError );
        double cellsize = Double.valueOf( dimensions[ 0 ].toString() ).doubleValue();
        double weightIntersect = 1.0d;
        double weightFactor = 1.0d;
        Grid2DSquareCellAbstract dummyGrid = null;
        long nrows = grid2DSquareCell.getNrows( handleOutOfMemoryError );
        long ncols = grid2DSquareCell.getNcols( handleOutOfMemoryError );
        File file = new File(
                dataDirectory,
                grid2DSquareCell.getName( handleOutOfMemoryError ) );
        //int distances = 2;
        ESRIAsciiGridExporter eSRIAsciiGridExporter = new ESRIAsciiGridExporter();
        ImageExporter imageExporter = new ImageExporter();
        for ( int distances = 2; distances <= 8; distances *=2 ) {
            double distance = cellsize * ( double ) distances;
            Grid2DSquareCellAbstract[] metrics1 = getMetrics1(
                    grid2DSquareCell,
                    distance,
                    weightIntersect,
                    weightFactor,
                    grid2DSquareCellDoubleFactory,
                    grid2DSquareCellIntFactory,
                    handleOutOfMemoryError );
            for ( int i = 0; i < metrics1.length; i ++ ) {
                mask(
                        metrics1[ i ],
                        0L,
                        0L,
                        ( long ) ( nrows - 1 ),
                        ( long ) ( distances - 1 ),
                        handleOutOfMemoryError );
                // mask right
                mask(
                        metrics1[ i ],
                        0L,
                        ncols - distances,
                        nrows - 1,
                        ncols - 1,
                        handleOutOfMemoryError );
                // mask top
                mask(
                        metrics1[ i ],
                        0L,
                        0L,
                        distances - 1,
                        ncols - 1,
                        handleOutOfMemoryError );
                // mask bottom
                mask(
                        metrics1[ i ],
                        nrows - distances,
                        0,
                        nrows - 1,
                        ncols - 1,
                        handleOutOfMemoryError );
                file =  new File(
                        dataDirectory,
                        metrics1[ i ].getName( handleOutOfMemoryError ) + "_" + distances + ".asc" );
                eSRIAsciiGridExporter.toAsciiFile(
                        metrics1[ i ],
                        file,
                        new BigDecimal(-9999.0),
                        handleOutOfMemoryError );
                file = new File(
                        dataDirectory,
                        metrics1[ i ].getName( handleOutOfMemoryError ) + "_" + distances + ".png" );
                imageExporter.toGreyScaleImage(
                        //imageExporter.toColourImage(
                        metrics1[ i ],
                        file,
                        "PNG",
                        handleOutOfMemoryError );
            }
        }
    }
    
}
